
require 'upload_module'
require 'validating_base'

class SearchBox < ValidatingBase
  attr_accessor :text
end

class PartnersController < ApplicationController
  include UploadModule
  before_filter :login_required
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  # 検索フォームの表示
  def index
  end
  
  def search
    @search = SearchBox.new(params[:search])
    if !@search.text || @search.text.strip == ''
      @search.errors.add_to_base '検索テキストを指定してください。'
      render :action => 'index'
      return
    end

    text = '%' + norm(@search.text.strip) + '%' # PostgreSQLの場合   TODO:fixme
    @partners = Partner.find :all,
                  :conditions => ['name LIKE ? OR kana_name LIKE ? OR remarks LIKE ?',
                                  text, text, text],
                  :limit => 40
  end

  def show
    @partner = Partner.find(params[:id])
  end

  def new
    @partner = Partner.new
  end
  
  # 10刻みで加算する
  def get_next_zaimu_id
    cs = CompanySetting.find(:first)
    raise if !cs
    while Partner.find_by_zaimu_id(cs.partner_counter)
      cs.partner_counter += 10 - (cs.partner_counter % 10)
    end
    r = cs.partner_counter
    cs.partner_counter += 10
    return r
  end
  private :get_next_zaimu_id
  
  # フォームデータを正規化
  def norm_params(pa)
    return pa.merge(
      :name => norm(pa[:name]),
      :kana_name => norm(pa[:kana_name]),
      :remarks => norm(pa[:remarks])
    )
  end
  private :norm_params
  
  # 新しく作成
  def create
    @partner = Partner.new(norm_params(params[:partner]))
    @partner.zaimu_id = get_next_zaimu_id() if @partner.zaimu_id.to_i == 0
    if @partner.save
      flash[:notice] = '取引先は、正常に作成されました。'
      redirect_to :action => 'show', :id => @partner
    else
      render :action => 'new'
    end
  end
  
  # 編集画面の表示
  def edit
    @partner = Partner.find(params[:id])
  end
  
  # 更新
  def update
    @partner = Partner.find(params[:id])
    if @partner.update_attributes(norm_params(params[:partner]))
      flash[:notice] = 'Partner was successfully updated.'
      redirect_to :action => 'show', :id => @partner
    else
      render :action => 'edit'
    end
  end

  def destroy
    partner = Partner.find(params[:id])
    s =  "取引先 #{partner.name} は、正常に削除されました。"
    partner.destroy
    flash[:notice] = s
    redirect_to :action => 'index'
  end
end
