
require 'validating_base'

class SearchBox < ValidatingBase
  attr_accessor :text, :code
end

# ユーザ
class UsersController < ApplicationController

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }
  
  # 検索画面
  def index
  end
  
  # 検索結果
  def search
    @search = SearchBox.new(params[:search])
    if @search.text.to_s.strip == '' && @search.code.to_s.strip == ''
      @search.errors.add_to_base '検索テキストを指定してください。'
      render :action => 'index'
      return
    end
    
    c = []; v = []
    if @search.text.to_s.strip != ''
      text = '%' + norm(@search.text.strip) + '%' # PostgreSQL限定。  TODO:fixme
      c << '(name LIKE ? OR kana_name LIKE ?)'
      v << text << text
    end
    if @search.code.to_s.strip != ''
      c << 'worker_code = ?'
      v << @search.code
    end
    @users = User.find :all,
                       :conditions => [c.join(' AND '), *v],
                       :limit => 40
  end
  
  def list
    @worker_pages, @workers = paginate :users, :per_page => 10
  end

  def show
    @worker = User.find(params[:id])
  end

  def new
    @worker = User.new
  end

  def create
    @worker = User.new(params[:worker])
    if @worker.save
      flash[:notice] = 'Worker was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @worker = User.find(params[:id])
  end

  def update
    @worker = User.find(params[:id])
    if @worker.update_attributes(params[:worker])
      flash[:notice] = 'Worker was successfully updated.'
      redirect_to :action => 'show', :id => @worker
    else
      render :action => 'edit'
    end
  end

  def destroy
    User.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
