
require 'csv'
require './import_common.rb'

establish_connection 'import_banks.log'

require '../app/models/bank'
require '../app/models/bank_branch'
require '../app/models/currency'

def import_banks
  data = sjis2utf8(File.read('banks-20060119.csv'))

  Bank.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1
      (print '.'; STDOUT.flush) if (idx % 100) == 0
      bank_id = line[0].to_i
      b = Bank.find(:first, :conditions => ['id = ?', bank_id])
      if !b
        b = Bank.new(:name => line[3].strip,
                     :kana_name => line[2].strip)
        b.id = bank_id
        b.save!
      end
      branch_code = line[1].to_i
      branch = BankBranch.find(:first,
                  :conditions => ['bank_id = ? AND branch_code = ?', bank_id, branch_code])
      if !branch
        r = BankBranch.new(:bank => b,
                           :branch_code => branch_code,
                           :name => line[5].strip,
                           :kana_name => line[4].strip)
        r.save!
      end
    }
  } # of transaction
end

# 通貨の設定。ユーザが自分でマスタを設定することはない。
def setup_currencies
  list = [
    ['JPY', '円', 0],
    ['USD', 'US$', 2],
    ['EUR', 'EURO', 2]
  ]
  Currency.transaction {
    list.each {|data|
      if cur = Currency.find_by_code(data[0])
        cur.name = data[1]
        cur.code = data[0]
        cur.exponent = data[2]
      else
        cur = Currency.new(:name => data[1], :code => data[0], :exponent => data[2])
      end
      cur.save!
    }
  }
end

import_banks()
setup_currencies()
