
# 財務大将から取引先をインポートする。
# 銀行口座の登録はすべて財務大将側で行い、
#     見える購買システムでは、親コードへの紐付け、個人旅客の不可視だけおこなう。

# 取引先の名称、コメントなどは購買システムで更新する。インポート時に上書きしない。

# 銀行口座のある相手先は、
#   bank_accounts にレコードがあれば更新
#   なければpartnerと二つレコードを作る
# 銀行口座のない相手先は、
#   partnerにレコードがあれば更新、なければ作る

require 'csv'
require './import_common.rb'

establish_connection 'import_partners.log'

require '../app/models/partner'
require '../app/models/bank_account'
require '../app/models/bank_branch'
require '../app/models/bank'
require '../app/models/attach_kind'

# Unicodeの正規化
def norm(s)
  s ? ActiveSupport::Multibyte::Handlers::UTF8Handler.normalize(s, :kc) : nil
end

# 取引先を探す。
# @return
#   見つからないときはnil
def find_partner(zaimu_id)
  partner = Partner.find_by_zaimu_id(zaimu_id)
  if !partner && (zaimu_id % 10) != 0
    # 末尾0で登録がないか探してみる
    partner = Partner.find_by_zaimu_id((zaimu_id / 10) * 10)
  end
  return partner
end

# 取引先レコードを探す・作る
def create_partner(line)
  zaimu_id = line[0].to_i
  if !(partner = find_partner(zaimu_id))
    partner = Partner.new(:name => norm(line[1].split(/[\:：]/u, 2)[0]),
                            :kana_name => (line[2] || ""),
                            :zaimu_id => zaimu_id,
                            :remarks => norm(line[1].split(/[\:：]/u, 2)[1] || ""))
    partner.save!
  end
  return partner
end

# 銀行を探す。見つからないときはエラー
def find_bank(line)
  bank_code = line[29].to_i
  branch_code = line[31].to_i
  branch = BankBranch.find_by_code(bank_code, branch_code)
  raise "bank not found: (#{bank_code}, #{branch_code})" if !branch
=begin
  if branch.bank.kana_name != line[30] || branch.kana_name != line[32]
    raise "bank (#{bank_code}, #{branch_code}): account = (#{line[30]}, #{line[32]}), but bank = (#{branch.bank.kana_name}, #{branch.kana_name})"
  end
=end
  return branch
end

def import_partners
  data = sjis2utf8(File.read('partners.csv'))
  
  Partner.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx == 1
      zaimu_id = line[0].to_i
      next if zaimu_id == 0
      next if zaimu_id >= 120000 && zaimu_id <= 129999 # 個人旅客
      next if zaimu_id >= 170000 && zaimu_id <= 179999 # 個人旅客2

      up = false
      bank_ac = BankAccount.find_by_id(line[0].to_i)
      if !bank_ac && (line[34].to_i != 0 && line[35].to_s.strip != '')   # 口座番号
        branch = find_bank(line)
        bank_ac = BankAccount.find(:first,
                   :conditions => ['bank_branch_id = ? AND ac_type = ? AND ac_number = ?',
                                   branch.id, line[33].to_i, line[34].to_i])
        if !bank_ac
          bank_ac = BankAccount.new()
          bank_ac.id = zaimu_id
          up = true
        end
      end
    
      if bank_ac
        # 口座の更新
        branch = find_bank(line) if !branch
        if bank_ac.bank_branch_id != branch.id ||
                bank_ac.ac_type != line[33].to_i ||
                bank_ac.ac_number != line[34].to_i ||
                bank_ac.kana_name != line[35].strip
          up = true
          bank_ac.bank_branch = branch
          bank_ac.ac_type = line[33]
          bank_ac.ac_number = line[34]
          bank_ac.kana_name = line[35].strip
        end
      
        if !bank_ac.partner
          up = true
          bank_ac.partner = create_partner(line)
        end
      
        bank_ac.save! if up
      else
        # 口座のない取引先
        create_partner(line)
      end
    }
  } # of transaction
end

def setup_attach_kind
  list = [
    ["業態票", 5, 0],
    ["振込口座登録票", 7, 0],
    ["取引基本契約", 10, 0],
    ["見積書", 20, 1],
    ["調達依頼書", 25, 1],
    ["稟議書", 27, 1],
    ["注文書 (控)", 30, 1],
    ["請書", 40, 1],
    ["納品書", 50, 1],
    ["受領書 (控)", 60, 1],
    ["請求書", 70, 1],
    ["写真", 75, 1],
    ["その他", 99, 1],
  ]
  AttachKind.transaction {
    list.each {|line|
      r = AttachKind.new(:name => line[0], :sort => line[1], :enable_invoice => line[2])
      r.save!
    }
  }
end

import_partners()
#setup_attach_kind()
