
# 勘定科目のインポート
# 財務大将から

require 'csv'
require './import_common.rb'

establish_connection 'import_titles.log'

require '../app/models/account_title'
require '../app/models/control_division'
require '../app/models/division'
require "../app/models/kubun"
require "../app/models/general_ledger"
require "../app/models/work_action"
require "../app/models/user"

def import_titles
  data = sjis2utf8(File.read('kanjo.csv'))
  hojo_data = sjis2utf8(File.read('hojo_kamoku.csv'))

  title_hash = {}

  idx = 0
  CSV::Reader.parse(data) {|line|
    idx += 1; next if idx == 1
    next if line[1].to_i > 999  # 科目は3桁
    next if line[0].to_i == 2   # 合計科目は取り込まない
    title_hash[line[1].to_i] = {:main => line, :suppl => {}}
  }
  idx = 0
  CSV::Reader.parse(hojo_data) {|line|
    idx += 1; next if idx == 1
    next if line[1].to_i > 999  # 科目は3桁
    # hojo_code, name, kana_name, start_day, end_day
    title_hash[line[1].to_i][:suppl][line[3].to_i] = line
  }

  AccountTitle.transaction {
    title_hash.each {|code, v|
      if v[:suppl].size > 0
        # 補助コードあり
        v[:suppl].each {|suppl_code, suppl_v|
          r = find_account_title(code, suppl_code) || AccountTitle.new(
                         :account_code => code,
                         :suppl_code => suppl_code,
                         :purchase_active => 1
          )
          r.name = "#{v[:main][2]}/#{suppl_v[4]}"
          r.kana_name = suppl_v[5]
          r.main_name = v[:main][2]
          r.start_day = suppl_v[9]
          r.end_day = suppl_v[10]
          r.vat_code = suppl_v[8]
          r.save!
        }
      else 
        # 補助コードなし
        r = find_account_title(code, 0) || AccountTitle.new(
                :account_code => code,
                :suppl_code => 0,
                :purchase_active => 1
        )
        r.name = v[:main][2]
        r.kana_name = v[:main][3]
        r.main_name = r.name
        r.start_day = v[:main][14]
        r.end_day = v[:main][15]
        r.vat_code = v[:main][9]
        r.save!
      end
    }
  }
end

def import_control_divisions
  data = sjis2utf8(File.read('control_divisions.csv'))
  ControlDivision.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx == 1
      raise if line[0].to_i == 0
      r = ControlDivision.find(:first, :conditions=>["id = ?", line[0].to_i]) || 
          ControlDivision.new(:id => line[0].to_i)
      r.name = line[1]
      r.save!
    }
  }
end

def import_divisions
  data = sjis2utf8(File.read('bumon.csv'))
  Division.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx == 1
      next if line[0].to_i != 0 && line[0].to_i != 1
      r = find_division_by_code(line[1].strip) ||
              Division.new(
                  :division_code => line[1].strip,
                  :purchase_active => 1)
      r.attributes = {
        :name => line[2],
        :kana_name => (line[3] || ""),
        :start_day => line[4],
        :end_day => line[5],
      }
      if !r.control_division_id
        r.control_division_id = (line[6] ? line[6].to_i : 1)
      end
      r.save!
    }
  }
end

def create_kubun
  kubun = Kubun.new(:name => "estimate")
  kubun.save!
end

def import_action_and_yosan
  ['jigyouhi_yosan.csv', 'kyuuyo.csv', 'lease.csv'].each {|filename|
    data = sjis2utf8(File.read(filename))

  GeneralLedger.transaction {
    kubun = Kubun.find(:first, :conditions => ["name = ?", "estimate"])
    raise if !kubun

    idx = 0
    days = []
    CSV::Reader.parse(data) {|line|
      idx += 1
      if idx == 1
        days = (0..5).collect {|i| line[16 + i]}
        next
      end

      wa = WorkAction.new(:name => line[0] || "")
      wa.save!

      div = find_division_by_code line[4]
      raise "div = #{line[4]}" if !div
      (0..5).each {|i|
        gl = GeneralLedger.new(
                 :date => days[i],
                 :kubun_id => kubun.id,
                 :division => div,
                 :account_title => find_account_title(line[7].to_i, line[8].to_i),
                 :work_action => wa,
                 :amount => (line[16 + i].to_f + 0.5).to_i * 1000,
                 :remarks => line[0] || "")
        gl.save!
      }
    }
  }
  }
end

# 社員マスタの取り込み。
# 財務大将の社員マスタでは、所属部署がないので、ITFlowから取り込む。
def import_workers
  data = sjis2utf8 File.read('SyainMST.csv')
  
  User.transaction {
    idx = 0
    CSV::Reader.parse(data) {|line|
      idx += 1; next if idx <= 4
      r = User.find_by_worker_code(line[0].to_i) ||
          User.new(:worker_code => line[0].to_i) 
      r.attributes = {
        :name => line[1],
        :kana_name => line[2],
        :division => find_division_by_code(line[3]),
        :email => line[6],
      }
      # 同一人物で社員コードが違うことがある。何だそれ。
      if r.email
        s = User.find :first,
              :conditions => ['email = ? AND worker_code <> ?', r.email, r.worker_code]
        next if s
      end
      begin
        r.save!
      rescue
        puts "error => ", r.inspect
        raise
      end
    }
  }
end

#import_titles()
#import_control_divisions()
#import_divisions()
#create_kubun()
#import_action_and_yosan()
import_workers()
